(define-module (ui ldap-groups ajax)
    :use-module (alterator woo)
    :use-module (alterator ajax)
    :use-module (alterator str)
    :use-module (srfi srfi-1)
    :use-module (srfi srfi-11)
    :use-module (srfi srfi-2)
    :use-module (alterator effect)
    :use-module (alterator plist)
    :export (init))
;;;

(define *rootdn* "")
(define *rootpw* "")
(define *bind_info* "present")
(define *host* "localhost")
(define *ldapbase* "")
(define *mode* "local")
(define *group* #f)
(define *netcmdplus* #f)

;;; members
(define (update-member-list . data)
  (let ((group (if (pair? data) (car data) (or (form-value "group") *group*))))
	(form-update-enum "member_in" (woo "member_list_in" "/ldap-groups" 'group group))
	(form-update-enum "member_out" (woo "member_list_out" "/ldap-groups" 'group group))))

(define (ui-member-add)
  (let ((group (or (form-value "group") *group*))
        (member_out (form-value "member_out")))
    (and (string? group) (string? member_out)
	 (catch/message
	   (lambda()
	     (woo "member_add"
	      "/ldap-groups"
		  'language (form-value "language")
		  'group group
		  'member_out member_out)
	     (update-member-list group))))))

(define (ui-member-del)
  (let ((group (or (form-value "group") *group*))
        (member_in (form-value "member_in")))
    (and (string? group) (string? member_in)
	 (catch/message
	   (lambda()
	     (woo "member_del"
	       "/ldap-groups"
		  'language (form-value "language")
		  'group group
		  'member_in member_in)
	     (update-member-list group))))))

;;; emails
(define (update-email-list . data)
  (let ((group (if (pair? data) (car data) (or (form-value "group") *group*))))
	(form-update-enum "email" (woo "email_list" "/ldap-groups" 'group group))))

(define (ui-email-add)
  (let ((group (or (form-value "group") *group*))
        (new_email (form-value "new_email")))
    (and (string? group) (string? new_email)
	 (catch/message
	   (lambda()
	     (woo "email_add"
	      "/ldap-groups"
		  'language (form-value "language")
		  'group group
		  'new_email new_email)
	     (form-update-value "new_email" "")
	     (update-email-list group))))))

(define (ui-email-del)
  (let ((group (or (form-value "group") *group*))
        (email (form-value "email")))
    (and (string? group) (string? email)
	 (catch/message
	   (lambda()
	     (woo "email_del"
	      "/ldap-groups"
		  'language (form-value "language")
		  'group group
		  'email email)
	     (update-email-list group))))))

;;; mapping
(define (map-enabled)
  ;; TODO: don't work at initial field fill
  (form-update-activity '("unix_gid") #t)
  ;;(form-value "has_unix_map"))
  ;;(form-value "has_samba_map"))
  (form-update-activity '("samba_name" "has_samba_map") (not (equal? *mode* "ad"))))

(define (fill-mapping group)
    (form-update-enum "unix_gid" (woo-list "/ldap-groups/system_groups" 'language (form-value "language")))
    (form-update-value-list
      '("has_unix_map" "unix_gid" "has_samba_map" "samba_name")
      (woo-read-first "/ldap-groups/mapping" 'group group))
    (map-enabled)
)

(define (apply-mapping group)
    (apply woo-write "/ldap-groups/mapping" 
        'language (form-value "language")
        'group group
        (form-value-list '("has_unix_map" "unix_gid" "has_samba_map" "samba_name"))))

;;; groups
(define (update-groups . data)
  (let ((group (if (pair? data) (car data) (form-value "group"))))
    (if (not (string-contains group ";"))
		(begin
		  (set! *group* group)
		  (form-update-visibility "update_message" #f)
		  (form-update-value "group_name" group)
		  (woo "member_reset" "/ldap-groups" 'group group)
		  (woo "email_reset" "/ldap-groups" 'group group)
		  (update-member-list group)
		  (fill-mapping group)
		  (if (equal? *mode* "ldap")
			  (update-email-list group)))
		(set! *group* #f))))

(define (ui-group-del)
  (let ((group (or (form-value "group") *group*)))
    (and (string? group)
	 (catch/message
	   (lambda()
	     (woo-delete "/ldap-groups" 'group group 'language (form-value "language"))
	     (ui-init))))))

(define (ui-group-add)
  (let ((new_group (form-value "new_group")))
    (and (string? new_group)
	 (catch/message
	   (lambda()
	     (woo-new "/ldap-groups" 'new_group new_group 'language (form-value "language"))
	     (ui-init)
	     (form-update-value "group" new_group)
	     (update-groups new_group))))))

(define (ui-group-save)
  (let ((group (or (form-value "group") *group*)))
    (and (string? group)
	 (catch/message
	   (lambda()
	     (form-update-visibility "update_message" #f)
	     (woo "member_commit"
	      "/ldap-groups" 'language (form-value "language") 'group group)
	    (if (equal? *mode* "ldap")
	     (woo "email_commit" "/ldap-groups" 'language (form-value "language") 'group group))
        (apply-mapping group)
	    (update-groups group)
	    (form-update-visibility "update_message" #t))))))

(define (local_s)
    (form-update-visibility "local_src" #t)
    (form-update-visibility "ldap_src" #f)
    (form-update-visibility "mail_h" #f)
    (form-update-visibility "mail_div" #f))

(define (ldap_s)
    (form-update-visibility "local_src" #f)
    (form-update-visibility "ldap_src" #t)
    (form-update-visibility "mail_h" #t)
    (form-update-visibility "mail_div" #f))

(define (select_other)
    (form-update-visibility "need_bind_info" #f)
    (form-update-visibility "main_area" #f)
    (form-update-visibility "addgrp_area" #f)
    (form-update-visibility "info_area" #f)
    (form-update-visibility "select_area" #t)
	(form-update-value "use_src" *mode*)
	(refresh-src *mode*))

(define (return_to_list)
    (form-update-visibility "need_bind_info" #f)
    (form-update-visibility "main_area" #t)
    (form-update-visibility "addgrp_area" #t)
    (form-update-visibility "info_area" #t)
    (form-update-visibility "select_area" #f)
)

(define (other_selected)
  (catch/message
   (lambda()
	 (let ((src (form-value "use_src")))
	   (woo-new "/ldap-groups/source"
				'newmode src
				'new_base (case (string->symbol src)
							((ldap) (form-value "rem_base_src"))
							((ad) (form-value "ad_base_src"))
							(else ""))
				'host (form-value "rem_host")
				'language (form-value "language")))))
  (ui-init))

(define (bind_info)
    (set! *rootpw* "")
    (form-update-value "bind_rootpw" *rootpw*)
    (form-update-visibility "need_bind_info" #t)
    (form-update-visibility "main_area" #f)
    (form-update-visibility "addgrp_area" #f)
    (form-update-visibility "info_area" #f)
    (form-update-visibility "select_area" #f)
)

(define (main_mode)
    (form-update-visibility "need_bind_info" #f)
    (form-update-visibility "select_area" #f)
    (form-update-visibility "main_area" #t)
    (form-update-visibility "addgrp_area" #t)
)

(define (check_mode)
  (let* ((data (woo-list "/ldap-groups/mode" )))
	(set! *bind_info* (woo-get-option (car data) 'bind_info))
	(set! *rootdn* (woo-get-option (car data) 'rootdn))
	(set! *rootpw* (woo-get-option (car data) 'rootpw))
	(set! *mode* (woo-get-option (car data) 'mode))
	(set! *ldapbase* (woo-get-option (car data) 'base))
	(set! *host* (woo-get-option (car data) 'host))
	(set! *netcmdplus* (woo-get-option (car data) 'netcmdplus_available)))
  (case (string->symbol *mode*)
	((ldap) (ldap_s))
	((ad) (form-update-value "base_type_" "Active Directory")
	 (form-update-visibility "base_type_" #t)
	 (ldap_s))
	((krb5) (set! *mode* "localldap")
	 (form-update-visibility "base_type_" #f)
	 (ldap_s))
	(else (form-update-visibility "base_type_" #f)
		  (local_s)))
  (form-update-value "base_" *ldapbase*)
  (form-update-value "host_" *host*)
  (if (equal? *host* "localhost")
      (begin
        (form-update-value "bind_rootdn" *rootdn*))
      (begin
        (form-update-value "bind_rootdn" "")))
  (bind_request))

(define (bind_request)
    (if (or (equal? *bind_info* "present")
			(equal? *mode* "ad"))
		(begin
		  (main_mode))
		(begin
		  (bind_info))))

(define (set_src)
  (refresh-src (car (string-cut-repeated
					 (or (form-value "use_src") "local") #\,))))

(define (refresh-src src)
  (case (string->symbol src)
	((ldap) (set_ldap_src))
	((localldap) (set_localldap_src))
	((ad) (set_ad_src))
	(else (set_local_src))))

(define (set_local_src)
  (form-update-visibility "new_base_src" #f)
  (form-update-visibility "rem_base_src_div" #f)
  (form-update-visibility "ad_options" #f)
  (form-update-activity "useit_btn" #t))

(define (set_ldap_src)
  (read_rem_bases)
  (form-update-visibility "new_base_src" #f)
  (form-update-visibility "rem_base_src_div" #t)
  (form-update-visibility "ad_options" #f))

(define (set_localldap_src)
  (let ((bases (woo-list "/ldap-groups/bases" 'mode "ldap")))
	(form-update-enum "new_base_src" bases)
	(form-update-value "new_base_src" *ldapbase*)
	(form-update-activity "useit_btn" (not (null? bases))))
  (form-update-visibility "new_base_src" #t)
  (form-update-visibility "rem_base_src_div" #f)
  (form-update-visibility "ad_options" #f))

(define (set_ad_src)
  (let ((bases (woo-list "/ldap-users/bases" 'mode "ad")))
	(form-update-enum "ad_base_src" bases)
	(form-update-value "ad_base_src" *ldapbase*)
	(form-update-activity "useit_btn" (not (null? bases))))
  (form-update-visibility "new_base_src" #f)
  (form-update-visibility "rem_base_src_div" #f)
  (form-update-visibility "ad_options" #t)
  (form-update-visibility "netcmdplus_unavail" (not *netcmdplus*)))

(define (read_rem_bases)
  (let ((bases (woo-list "/ldap-users/bases"
						 'mode "ldap"
						 'rem_host (form-value "rem_host"))))
	(form-update-enum "rem_base_src" bases)
	(form-update-value "rem_base_src" *ldapbase*)
	(form-update-activity "useit_btn" (not (null? bases)))))

(define (set_bind)
    (if (not (string-null? (form-value "bind_rootpw")))
       (begin
        (set! *rootpw* (form-value "bind_rootpw"))
        (set! *rootdn* (form-value "bind_rootdn"))
        (set! *bind_info* "present")))
        (woo "set_bind" "/ldap-groups"
            'mode "ldap"
            'base *ldapbase*
            'host *host*
            'rootpw *rootpw*
            'rootdn *rootdn*
            'bind_info *bind_info*
            'language (form-value "language") )
    (bind_request))

;;; interface
(define (ui-init)
    (check_mode)
    (form-update-value "new_group" "")
    (form-update-value "group_name" "")
    (form-update-enum "member_in" '())
    (form-update-enum "member_out" '())
    (form-update-value "new_email" "")
    (form-update-visibility "update_message" #f)
    (form-update-visibility "info_area" #t)
    (form-update-enum "group" 
	  (woo-list "/ldap-groups/" 
	  'language (form-value "language")))
)

(define (init)
    (ui-init)
    (form-bind "group" "change" update-groups)
    (form-bind "use_src" "change" set_src)
    (form-bind "rem_host" "change" read_rem_bases)
    (form-bind "member_add_button" "click" ui-member-add)
    (form-bind "member_del_button" "click" ui-member-del)
    (form-bind "email_add_button" "click" ui-email-add)
    (form-bind "email_del_button" "click" ui-email-del)
    (form-bind "group_save_button" "click" ui-group-save)
    (form-bind "group_del_button" "click" ui-group-del)
    (form-bind "group_add_button" "click" ui-group-add)
    (form-bind "select_btn" "click" select_other)
    (form-bind "useit_btn" "click" other_selected)
    (form-bind "ret_list" "click" return_to_list)
    (form-bind "bind_btn" "click" set_bind)
    (form-bind "has_unix_map" "change" map-enabled)
    (form-bind "has_samba_map" "change" map-enabled)
)
